@@ -7,10 +7,18 @@ use Data::Dump qw( dump );
 use Search::Query::Field::SQL;
 
 __PACKAGE__->mk_accessors(
-    qw( wildcard quote_fields default_field fuzzify fuzzify2 croak_on_error like )
+    qw(
+        wildcard
+        quote_fields
+        fuzzify
+        fuzzify2
+        like
+        quote_char
+        fuzzy_space
+        )
 );
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 =head1 NAME
 
@@ -68,9 +76,13 @@ Prepend and append wildcard() to all terms.
 
 The SQL reserved word for wildcard comparison. Default value is C<ILIKE>.
 
-=item croak_on_error
+=item quote_char
 
-Croak if any field validation fails.
+The string to use for quoting strings. Default is C<'>.
+
+=item fuzzy_space
+
+The string to use to pad fuzzified terms. Default is a single space C< >.
 
 =back
 
@@ -83,12 +95,17 @@ sub init {
     #carp dump $self;
     $self->{wildcard} ||= '%';
     $self->{quote_fields} = '' unless exists $self->{quote_fields};
-    $self->{default_field} ||= $self->{_parser}->default_field
-        || [ sort keys %{ $self->{_parser}->fields } ];
+    if ( !defined $self->parser->fields ) {
+        croak "You must set fields in the Search::Query::Parser";
+    }
+    $self->{default_field} ||= $self->parser->default_field
+        || [ sort keys %{ $self->parser->fields } ];
     if ( $self->{default_field} and !ref( $self->{default_field} ) ) {
         $self->{default_field} = [ $self->{default_field} ];
     }
     $self->{like} ||= 'ILIKE';
+    $self->{quote_char}  = q/'/ unless exists $self->{quote_char};
+    $self->{fuzzy_space} = ' '  unless exists $self->{fuzzy_space};
     return $self;
 }
 
@@ -112,7 +129,7 @@ sub stringify {
     foreach my $prefix ( '+', '', '-' ) {
         my @clauses;
         my $joiner = $op_map{$prefix};
-        next if not $tree->{$prefix};
+        next unless exists $tree->{$prefix};
         for my $clause ( @{ $tree->{$prefix} } ) {
             push( @clauses, $self->stringify_clause( $clause, $prefix ) );
         }
@@ -154,12 +171,20 @@ sub stringify_clause {
     my $clause = shift;
     my $prefix = shift;
 
-    return "(" . $self->stringify( $clause->{value} ) . ")"
-        if $clause->{op} eq '()';
+    if ( $clause->{op} eq '()' ) {
+        if ( $clause->has_children and $clause->has_children == 1 ) {
+            return $self->stringify( $clause->{value} );
+        }
+        else {
+            return "(" . $self->stringify( $clause->{value} ) . ")";
+        }
+    }
 
     # optional
     my $quote_fields = $self->quote_fields;
 
+    my $fuzzy_space = $self->fuzzy_space;
+
     # make sure we have a field
     my @fields
         = $clause->{field}
@@ -188,7 +213,7 @@ NAME: for my $name (@fields) {
         my $this_op;
 
         # whether we quote depends on the field (column) type
-        my $quote = $field->is_int ? "" : "'";
+        my $quote = $field->is_int ? "" : $self->quote_char;
 
         # fuzzy
         if ( $op =~ m/\~/ ) {
@@ -199,7 +224,8 @@ NAME: for my $name (@fields) {
                     $this_op = $field->fuzzy_not_op;
                 }
                 else {
-                    $this_op = ' ' . $field->fuzzy_not_op . ' ';
+                    $this_op
+                        = $fuzzy_space . $field->fuzzy_not_op . $fuzzy_space;
                 }
             }
 
@@ -209,7 +235,7 @@ NAME: for my $name (@fields) {
                     $this_op = $field->fuzzy_op;
                 }
                 else {
-                    $this_op = ' ' . $field->fuzzy_op . ' ';
+                    $this_op = $fuzzy_space . $field->fuzzy_op . $fuzzy_space;
                 }
             }
         }
@@ -241,14 +267,7 @@ NAME: for my $name (@fields) {
 
 sub _get_field {
     my $self  = shift;
-    my $name  = shift or croak "field name required";
-    my $field = $self->{_parser}->get_field($name);
-    if ( !$field ) {
-        if ( $self->croak_on_error ) {
-            croak "invalid field name: $name";
-        }
-        $field = $self->field_class->new( name => $name );
-    }
+    my $field = $self->SUPER::_get_field(@_);
 
     # fix up the operator based on our like() setting
     $field->fuzzy_op( $self->like ) if !$field->is_int;
@@ -257,15 +276,6 @@ sub _get_field {
     return $field;
 }
 
-sub _get_default_field {
-    my $self = shift;
-    my $field = $self->default_field || $self->{_parser}->default_field;
-    if ( !$field ) {
-        croak "must define a default_field";
-    }
-    return ref $field ? $field : [$field];
-}
-
 =head2 field_class
 
 Returns "Search::Query::Field::SQL".
