@@ -1,16 +1,19 @@
 package Search::Query::Dialect::SWISH;
-use Moo;
-extends 'Search::Query::Dialect';
+use strict;
+use warnings;
+use base qw( Search::Query::Dialect );
 use Carp;
 use Data::Dump qw( dump );
 use Search::Query::Field::SWISH;
-use Try::Tiny;
 
-our $VERSION = '0.290_02';
+our $VERSION = '0.07';
 
-has 'wildcard'       => ( is => 'rw', default => '*' );
-has 'fuzzify'        => ( is => 'rw' );
-has '+default_field' => ( is => 'rw', default => 'swishdefault' );
+__PACKAGE__->mk_accessors(
+    qw(
+        wildcard
+        fuzzify
+        )
+);
 
 =head1 NAME
 
@@ -36,10 +39,9 @@ methods are documented here.
 
 =cut
 
-=head2 BUILD
-
-Sets SWISH-appropriate defaults.
+=head2 init
 
+Overrides base method and sets SWISH-appropriate defaults.
 Can take the following params, also available as standard attribute
 methods.
 
@@ -53,39 +55,29 @@ Default is '*'.
 
 If true, a wildcard is automatically appended to each query term.
 
-=item default_field
-
-Default is 'swishdefault'.
-
 =back
 
 =cut
 
-sub BUILD {
+sub init {
     my $self = shift;
 
+    $self->SUPER::init(@_);
+
     #carp dump $self;
-
-    # make sure we have our default field defined amongst all parser fields.
-    my $swishdefault_field = try {
-        $self->parser->get_field('swishdefault');
+    $self->{wildcard} = '*';
+    if ( $self->parser->fields ) {
+        $self->{default_field} ||= $self->parser->default_field
+            || [ sort keys %{ $self->parser->fields } ];
     }
-    catch {
-        carp "swishdefault not amongst parser fields: $_";
-    };
-    if ( !$swishdefault_field ) {
-        $self->parser->set_field( 'swishdefault',
-            Search::Query::Field::SWISH->new( name => 'swishdefault' ) );
+    else {
+        $self->{default_field} ||= $self->parser->default_field
+            || 'swishdefault';
     }
-
-    #carp "swishdefault_field=" . dump($swishdefault_field);
-
     if ( $self->{default_field} and !ref( $self->{default_field} ) ) {
         $self->{default_field} = [ $self->{default_field} ];
     }
 
-    #carp dump $self;
-
     return $self;
 }
 
@@ -96,9 +88,9 @@ Returns the Query object as a normalized string.
 =cut
 
 my %op_map = (
-    '+' => ' AND ',
-    ''  => ' OR ',
-    '-' => ' ',
+    '+' => 'AND',
+    ''  => 'OR',
+    '-' => 'NOT',
 );
 
 sub stringify {
@@ -115,10 +107,10 @@ sub stringify {
         }
         next if !@clauses;
 
-        push @q, join( $joiner, grep { defined and length } @clauses );
+        push @q, join( " $joiner ", grep { defined and length } @clauses );
     }
 
-    return join " ", @q;    # Swish-e defaults to AND but we can't predict.
+    return join " AND ", @q;
 }
 
 sub _doctor_value {
@@ -126,8 +118,6 @@ sub _doctor_value {
 
     my $value = $clause->{value};
 
-    return $value unless defined $value;
-
     if ( $self->fuzzify ) {
         $value .= '*' unless $value =~ m/[\*\%]/;
     }
@@ -154,12 +144,13 @@ sub stringify_clause {
     #warn "prefix = '$prefix'";
 
     if ( $clause->{op} eq '()' ) {
-        my $str = $self->stringify( $clause->{value} );
-        if ( $prefix eq '-' ) {
-            return "NOT ($str)";
+        if ( $clause->has_children and $clause->has_children == 1 ) {
+            return $self->stringify( $clause->{value} );
         }
         else {
-            return "($str)";
+            return
+                ( $prefix eq '-' ? 'NOT ' : '' ) . "("
+                . $self->stringify( $clause->{value} ) . ")";
         }
     }
 
@@ -167,7 +158,7 @@ sub stringify_clause {
     my @fields
         = $clause->{field}
         ? ( $clause->{field} )
-        : ( @{ $self->get_default_field } );
+        : ( @{ $self->_get_default_field } );
 
     # what value
     my $value
@@ -185,23 +176,15 @@ sub stringify_clause {
     if ( $prefix eq '-' ) {
         $op = '!' . $op;
     }
-    if ( defined $value and $value =~ m/\%/ ) {
+    if ( $value =~ m/\%/ ) {
         $op = $prefix eq '-' ? '!~' : '~';
     }
 
-    my $quote       = $clause->quote || '';
-    my $left_quote  = $quote;
-    my $right_quote = $quote;
-    my $proximity   = $clause->proximity || '';
-    if ($proximity) {
-        $value =~ s/\s+/ NEAR$proximity /g;
-        $left_quote  = '(';
-        $right_quote = ')';
-    }
+    my $quote = $clause->quote || '';
 
     my @buf;
 NAME: for my $name (@fields) {
-        my $field = $self->get_field($name);
+        my $field = $self->_get_field($name);
 
         if ( defined $field->callback ) {
             push( @buf, $field->callback->( $field, $op, $value ) );
@@ -213,32 +196,20 @@ NAME: for my $name (@fields) {
         # invert fuzzy
         if ( $op eq '!~' ) {
             $value .= $wildcard unless $value =~ m/\Q$wildcard/;
-            push(
-                @buf,
-                join( '',
-                    'NOT ', $name,
-                    '=',    qq/${left_quote}${value}${right_quote}/ )
-            );
+            push( @buf,
+                join( '', 'NOT ', $name, '=', qq/$quote$value$quote/ ) );
         }
 
         # fuzzy
         elsif ( $op eq '~' ) {
             $value .= $wildcard unless $value =~ m/\Q$wildcard/;
-            push(
-                @buf,
-                join( '',
-                    $name, '=', qq/${left_quote}${value}${right_quote}/ )
-            );
+            push( @buf, join( '', $name, '=', qq/$quote$value$quote/ ) );
         }
 
         # invert
-        elsif ( defined $value and $op eq '!=' ) {
-            push(
-                @buf,
-                join( '',
-                    'NOT ', $name,
-                    '=',    qq/${left_quote}${value}${right_quote}/ )
-            );
+        elsif ( $op eq '!=' ) {
+            push( @buf,
+                join( '', 'NOT ', $name, '=', qq/$quote$value$quote/ ) );
         }
 
         # range
@@ -281,18 +252,9 @@ NAME: for my $name (@fields) {
             );
         }
 
-        # null query
-        elsif ( !defined $value ) {
-            croak "SWISH dialect does not support NULL query term";
-        }
-
         # standard
         else {
-            push(
-                @buf,
-                join( '',
-                    $name, '=', qq/${left_quote}${value}${right_quote}/ )
-            );
+            push( @buf, join( '', $name, '=', qq/$quote$value$quote/ ) );
         }
     }
     my $joiner = $prefix eq '-' ? ' AND ' : ' OR ';
